"use strict";

var Promise = require("bluebird");
var xss = require('xss');

var AbstractGuiElement = require('../AbstractGuiElement');
var GuiConnector = require('../../GuiConnector');
var ValidationError = require('../../ValidationError');

var Functions = require('../../Functions');
// noinspection JSUnusedLocalSymbols
var logger = require('../../logger');

var guiUtils = new (require('../leftPanel/GuiUtils'))();

/**
 *
 * @param params
 * @param {HTMLElement} params.element
 * @param {User} params.user
 * @param {Configuration} params.configuration
 * @param {ServerConnector} params.serverConnector
 *
 * @constructor
 * @extends {AbstractGuiElement}
 */
function EditUserDialog(params) {
  AbstractGuiElement.call(this, params);
  var self = this;
  self.setUser(params.user);

  $(self.getElement()).css({overflow: "hidden"});

  self.createGui();
  self.registerListenerType("onSave");
}

EditUserDialog.prototype = Object.create(AbstractGuiElement.prototype);
EditUserDialog.prototype.constructor = EditUserDialog;

/**
 *
 * @param {User} user
 */
EditUserDialog.prototype.setUser = function (user) {
  this._user = user;
  this.setIsNewUser(user.getLogin() === undefined);
};

/**
 *
 * @returns {User}
 */
EditUserDialog.prototype.getUser = function () {
  return this._user;
};

/**
 *
 * @param {boolean} isNewUser
 */
EditUserDialog.prototype.setIsNewUser = function (isNewUser) {
  this._isNewUser = isNewUser;
};

/**
 *
 * @returns {boolean}
 */
EditUserDialog.prototype.getIsNewUser = function () {
  return this._isNewUser;
};

/**
 *
 */
EditUserDialog.prototype.createGui = function () {
  var self = this;

  guiUtils.initTabContent(self);

  guiUtils.addTab(self, {
    name: "DETAILS",
    content: self.createGeneralTabContent()
  });
  guiUtils.addTab(self, {
    name: "GLOBAL PRIVILEGES",
    content: self.createPrivilegesTabContent()
  });
  guiUtils.addTab(self, {
    name: "PROJECT PRIVILEGES",
    content: self.createProjectsTabContent()
  });

  $(".nav-tabs > li > a", self.getElement()).bind("click", function () {
    //workaround for some css issues...
    $(".parentTabs", self.getElement()).css("top", "40px");
    $(".parentTabs", self.getElement()).css("bottom", "10px");
  });
};

/**
 *
 * @param value
 * @returns {string}
 */
function getStringIfDefined(value) {
  if (value === undefined) {
    return "";
  }
  return xss(value);
}

/**
 *
 * @returns {HTMLElement}
 */
EditUserDialog.prototype.createGeneralTabContent = function () {

  var self = this;

  var result = Functions.createElement({
    type: "div",
    style: "margin-top:10px;"
  });

  var table = Functions.createElement({
    type: "table",
    name: "detailsTable",
    className: "display",
    style: "width:100%"
  });
  result.appendChild(table);

  var menuRow = Functions.createElement({
    type: "div",
    className: "minerva-menu-row",
    style: "display:table-row; margin:10px"
  });
  result.appendChild(menuRow);

  var saveUserButton = Functions.createElement({
    type: "button",
    name: "saveUser",
    content: '<span class="ui-icon ui-icon-disk"></span>&nbsp;SAVE',
    onclick: function () {
      GuiConnector.showProcessing();
      return self.onSaveClicked().then(function () {
        return self.close();
      }).catch(GuiConnector.alert).finally(function () {
        GuiConnector.hideProcessing();
      });
    },
    xss: false
  });
  var cancelButton = Functions.createElement({
    type: "button",
    name: "cancelProject",
    content: '<span class="ui-icon ui-icon-cancel"></span>&nbsp;CANCEL',
    onclick: function () {
      return self.close();
    },
    xss: false
  });
  menuRow.appendChild(saveUserButton);
  menuRow.appendChild(cancelButton);

  return result;

};

/**
 *
 * @returns {HTMLElement}
 */
EditUserDialog.prototype.createPrivilegesTabContent = function () {
  var self = this;
  var result = Functions.createElement({
    type: "div",
    style: "margin-top:10px;"
  });
  result.appendChild(self._createPrivilegesTable());
  return result;
};

/**
 *
 * @returns {HTMLElement}
 * @private
 */
EditUserDialog.prototype._createPrivilegesTable = function () {
  var result = Functions.createElement({
    type: "div",
    style: "margin-top:10px;"
  });

  var privilegesTable = Functions.createElement({
    type: "table",
    name: "privilegesTable",
    className: "display",
    style: "width:100%"
  });
  result.appendChild(privilegesTable);

  return result;
};


/**
 *
 * @returns {HTMLElement}
 */
EditUserDialog.prototype.createProjectsTabContent = function () {
  var self = this;
  var result = Functions.createElement({
    type: "div",
    style: "margin-top:10px;"
  });
  result.appendChild(self._createProjectsTable());
  return result;
};

/**
 *
 * @returns {HTMLElement}
 * @private
 */
EditUserDialog.prototype._createProjectsTable = function () {
  var result = Functions.createElement({
    type: "div",
    style: "margin-top:10px;"
  });

  result.appendChild(Functions.createElement({
    type: "div",
    name: "defaultProjectsRow",
    style: "width:100%"
  }));

  result.appendChild(Functions.createElement({
    type: "table",
    name: "projectsTable",
    className: "display",
    style: "width:100%"
  }));

  return result;
};

/**
 *
 * @returns {Promise}
 */
EditUserDialog.prototype.init = function () {
  var self = this;

  var detailsTable = $("[name=detailsTable]", self.getElement())[0];

  $(detailsTable).DataTable({
    columns: [{
      title: "Name"
    }, {
      title: "Value"
    }],
    paging: false,
    ordering: false,
    searching: false,
    bInfo: false
  });

  self.initProjectsTab();
  self.initPrivilegesTab();

  return self.refresh().then(function () {
    $(window).trigger('resize');
  });
};

EditUserDialog.prototype.refresh = function () {
  var self = this;

  var dataTable = $("[name=detailsTable]", self.getElement()).DataTable();

  var data = [];

  var user = self.getUser();

  var readonly = '';
  if (user.getLogin() !== undefined) {
    readonly = ' readonly ';
  }

  data.push(['Login', '<input name="userLogin" value="' + getStringIfDefined(user.getLogin()) + '" ' + readonly + '/>']);
  data.push(['Password', '<input type="password" name="userPassword" value=""/>']);
  data.push(['Confirm password', '<input type="password" name="userPassword2" value=""/>']);
  data.push(['Name', '<input name="userName" value="' + getStringIfDefined(user.getName()) + '"/>']);
  data.push(['Surname', '<input name="userSurname" value="' + getStringIfDefined(user.getSurname()) + '"/>']);
  data.push(['Email', '<input name="userEmail" value="' + getStringIfDefined(user.getEmail()) + '"/>']);

  dataTable.clear().rows.add(data).draw();

  return self.refreshProjects().then(function () {
    return self.refreshPrivilegesTab();
  });
};

/**
 *
 */
EditUserDialog.prototype.initProjectsTab = function () {
  var self = this;

  var projectsTable = $("[name=projectsTable]", self.getElement())[0];

  var configuration = self.getConfiguration();
  var columns = self.createUserPrivilegeColumns();
  $(projectsTable).DataTable({
    columns: columns
  });
  $(projectsTable).on("click", "[name='project-privilege-checkbox']", function () {
    var data = $(this).attr("data").split("-");
    var privilegeType = data[0];
    var objectId = data[1];
    var value = $(this).is(":checked") ? 1 : 0;
    for (var i = 0; i < configuration.getPrivilegeTypes().length; i++) {
      var privilege = configuration.getPrivilegeTypes()[i];
      if (privilege.getName() === privilegeType) {
        self.getUser().setPrivilege({type: privilege, value: value, objectId: objectId});
      }
    }
  });

  $("[name='defaultProjectsRow']", self.getElement()).on("click", "[name='project-privilege-checkbox']", function () {
    var data = $(this).attr("data").split("-");
    var privilegeType = data[0];
    var objectId = data[1];
    var value = $(this).is(":checked") ? 1 : 0;
    for (var i = 0; i < configuration.getPrivilegeTypes().length; i++) {
      var privilege = configuration.getPrivilegeTypes()[i];
      if (privilege.getName() === privilegeType) {
        self.getUser().setPrivilege({type: privilege, value: value, objectId: objectId});
      }
    }
  });
};

/**
 *
 */
EditUserDialog.prototype.initPrivilegesTab = function () {
  var self = this;

  var privilegesTable = $("[name=privilegesTable]", self.getElement())[0];

  var columns = [{
    title: "Name"
  }, {
    title: "Value"
  }];

  $(privilegesTable).DataTable({
    columns: columns
  });

  var configuration = self.getConfiguration();
  $(privilegesTable).on("click", "[name='privilege-checkbox']", function () {
    var privilegeType = $(this).attr("data");
    for (var i = 0; i < configuration.getPrivilegeTypes().length; i++) {
      var privilege = configuration.getPrivilegeTypes()[i];
      if (privilege.getName() === privilegeType) {
        var value = $(this).is(":checked") ? 1 : 0;
        self.getUser().setPrivilege({type: privilege, value: value});
      }
    }
  });
  $(privilegesTable).on("change", "[name='privilege-int']", function () {
    var privilegeType = $(this).attr("data");
    var value = $(this).val();
    value = parseInt(value);
    if (Functions.isInt(value)) {
      for (var i = 0; i < configuration.getPrivilegeTypes().length; i++) {
        var privilege = configuration.getPrivilegeTypes()[i];
        if (privilege.getName() === privilegeType) {
          self.getUser().setPrivilege({type: privilege, value: value});
        }
      }
    }
  });

};

/**
 *
 */
EditUserDialog.prototype.refreshPrivilegesTab = function () {
  var self = this;

  var dataTable = $("[name=privilegesTable]", self.getElement()).DataTable();

  var data = [];
  var configuration = self.getConfiguration();
  for (var i = 0; i < configuration.getPrivilegeTypes().length; i++) {
    var privilege = configuration.getPrivilegeTypes()[i];
    var option;
    if (privilege.getObjectType() === null) {
      if (privilege.getValueType() === "boolean") {
        var checked = "";
        if (self.getUser().hasPrivilege(privilege)) {
          checked = "checked"
        } else if (this.getIsNewUser()) {
          option = this.getConfiguration().getOption('DEFAULT_' + privilege.getName());
          if (option !== null && option !== undefined) {
            if (option.getValue().toLowerCase() === "true") {
              self.getUser().setPrivilege({type: privilege});
              checked = "checked";
            }
          }
        }
        data.push([
          "<span>" + privilege.getCommonName() + "</span>",
          "<input type='checkbox' name='privilege-checkbox' data='" + privilege.getName() + "' " + checked + " />"
        ]);
      } else if (privilege.getValueType() === "int") {
        var value = self.getUser().getPrivilegeValue(privilege);
        if (this.getIsNewUser()) {
          option = this.getConfiguration().getOption('DEFAULT_' + privilege.getName());
          if (option !== null && option !== undefined) {
            value = parseInt(option.getValue());
            self.getUser().setPrivilege({type: privilege, value: value});
          }
        }
        data.push([
          "<span>" + privilege.getCommonName() + "</span>",
          "<input name='privilege-int' data='" + privilege.getName() + "' value='" + value + "' />"
        ]);

      }
    }
  }
  dataTable.clear().rows.add(data).draw();
};

/**
 *
 * @returns {Promise}
 */
EditUserDialog.prototype.refreshProjects = function () {
  var self = this;
  return self.getServerConnector().getProjects().then(function (projects) {
    return self.setProjects(projects);
  });
};

/**
 *
 * @param projects
 */
EditUserDialog.prototype.setProjects = function (projects) {
  var self = this;
  self._userByLogin = [];
  var columns = self.createUserPrivilegeColumns();
  var dataTable = $("[name='projectsTable']", self.getElement()).DataTable();
  var data = [], i;

  var rowData = self.projectToTableRow(null, columns);
  var defaultRow = $("[name='defaultProjectsRow']", self.getElement())[0];
  defaultRow.innerHTML = "";
  defaultRow.appendChild(Functions.createElement({
    type: "span",
    content: rowData[0],
    xss: false
  }));
  defaultRow.appendChild(Functions.createElement({type: "br"}));
  for (i = 1; i < columns.length; i++) {
    defaultRow.appendChild(Functions.createElement({
      type: "div",
      content: rowData[i] + columns[i].title,
      style: "float:left;padding:5px;",
      xss: false
    }));
  }
  defaultRow.appendChild(Functions.createElement({type: "br"}));
  defaultRow.appendChild(Functions.createElement({type: "hr"}));
  // data.push(rowData);
  for (i = 0; i < projects.length; i++) {
    var project = projects[i];
    rowData = self.projectToTableRow(project, columns);
    data.push(rowData);
  }
  dataTable.clear().rows.add(data).draw();
};

/**
 *
 * @param {Project} project
 * @param {Array} columns
 * @returns {Array}
 */
EditUserDialog.prototype.projectToTableRow = function (project, columns) {
  var user = this.getUser();
  var row = [];
  var id = null;
  var projectId = "<b>DEFAULT PRIVILEGE FOR NEW PROJECT</b>";
  if (project !== null) {
    id = project.getId();
    projectId = project.getProjectId();
  }

  row[0] = "<span data='" + id + "'>" + projectId + "</span>";

  for (var i = 1; i < columns.length; i++) {
    var privilege = columns[i].privilegeType;
    var checked = "";
    if (user.hasPrivilege(privilege, id)) {
      checked = "checked";
    } else if (this.getIsNewUser()) {
      var option = this.getConfiguration().getOption('DEFAULT_' + privilege.getName());
      if (option !== null && option !== undefined) {
        if (option.getValue().toLowerCase() === "true") {

          user.setPrivilege({type: privilege, objectId: id});
          checked = "checked";
        }
      }
    }
    row.push("<input type='checkbox' name='project-privilege-checkbox' data='" + privilege.getName() + "-" + id + "' " + checked + " />");
  }

  return row;
};


/**
 *
 */
EditUserDialog.prototype.destroy = function () {
  var self = this;
  var div = self.getElement();
  var usersTable = $("[name=projectsTable]", div)[0];
  if ($.fn.DataTable.isDataTable(usersTable)) {
    $(usersTable).DataTable().destroy();
  }

  var privilegesTable = $("[name=privilegesTable]", div)[0];
  if ($.fn.DataTable.isDataTable(privilegesTable)) {
    $(privilegesTable).DataTable().destroy();
  }

  var detailsTable = $("[name=detailsTable]", div)[0];
  if ($.fn.DataTable.isDataTable(detailsTable)) {
    $(detailsTable).DataTable().destroy();
  }


  if ($(div).hasClass("ui-dialog-content")) {
    $(div).dialog("destroy");
  }
};

/**
 *
 */
EditUserDialog.prototype.open = function () {
  var self = this;
  var div = self.getElement();
  var title = self.getUser().getLogin();
  if (title === undefined) {
    title = "NEW USER";
  }
  if (!$(div).hasClass("ui-dialog-content")) {
    $(div).dialog({
      dialogClass: 'minerva-edit-user-dialog',
      title: title,
      width: window.innerWidth / 2,
      height: window.innerHeight / 2
    });
  }
  $(div).dialog("open");
};

/**
 *
 * @returns {Promise}
 */
EditUserDialog.prototype.onSaveClicked = function () {
  var self = this;
  var user = self.getUser();
  return self.checkValidity().then(function () {
    user.setLogin(self.getLogin());
    user.setPassword(self.getPassword());
    user.setEmail(self.getEmail());
    user.setName(self.getName());
    user.setSurname(self.getSurname());
    if (self.getIsNewUser()) {
      return self.getServerConnector().addUser(user);
    } else {
      return self.getServerConnector().updateUser(user);
    }
  }).then(function () {
    return self.callListeners("onSave", user);
  });
};

/**
 *
 * @returns {Promise}
 */
EditUserDialog.prototype.checkValidity = function () {
  var self = this;
  var isValid = true;
  var error = "<b>Some data is missing.</b><ul>";
  if (self.getPassword() !== self.getPassword2()) {
    error += "<li>Password doesn't match</li>";
    isValid = false;
  }
  if (self.getLogin() === "" || self.getLogin() === undefined) {
    error += "<li>Login must not be empty</li>";
    isValid = false;
  }
  var promise = Promise.resolve();
  if (self.getUser().getLogin() !== self.getLogin()) {
    promise = ServerConnector.getUser(self.getLogin()).then(function (remoteUser) {
      if (remoteUser !== null) {
        error += "<li>Login already used</li>";
        isValid = false;
      }
    });
  }
  return promise.then(function () {
    if (isValid) {
      return Promise.resolve(true);
    } else {
      return Promise.reject(new ValidationError(error));
    }
  });
};

/**
 *
 * @returns {string}
 */
EditUserDialog.prototype.getPassword = function () {
  var self = this;
  return $("[name='userPassword']", self.getElement()).val();
};

/**
 *
 * @returns {string}
 */
EditUserDialog.prototype.getPassword2 = function () {
  var self = this;
  return $("[name='userPassword2']", self.getElement()).val();
};

/**
 *
 * @returns {string}
 */
EditUserDialog.prototype.getLogin = function () {
  var self = this;
  return $("[name='userLogin']", self.getElement()).val();
};

/**
 *
 * @returns {string}
 */
EditUserDialog.prototype.getEmail = function () {
  var self = this;
  return $("[name='userEmail']", self.getElement()).val();
};

/**
 *
 * @returns {string}
 */
EditUserDialog.prototype.getName = function () {
  var self = this;
  return $("[name='userName']", self.getElement()).val();
};

/**
 *
 * @returns {string}
 */
EditUserDialog.prototype.getSurname = function () {
  var self = this;
  return $("[name='userSurname']", self.getElement()).val();
};

/**
 *
 */
EditUserDialog.prototype.close = function () {
  var self = this;
  if ($(self.getElement()).hasClass("ui-dialog-content")) {
    //close it only if it wasn't destroyed
    $(self.getElement()).dialog("close");
  }
};

/**
 *
 * @returns {{title: string, privilegeType: PrivilegeType|undefined}[]}
 */
EditUserDialog.prototype.createUserPrivilegeColumns = function () {
  var self = this;

  if (self._userPrivilegeColumns !== undefined) {
    return self._userPrivilegeColumns;
  }

  var configuration = self.getConfiguration();
  self._userPrivilegeColumns = [{
    title: "ProjectId"
  }];
  var privilegeTypes = configuration.getPrivilegeTypes();
  for (var i = 0; i < privilegeTypes.length; i++) {
    var type = privilegeTypes[i];
    if (type.getObjectType() === "Project") {
      self._userPrivilegeColumns.push({
        "title": type.getCommonName(),
        privilegeType: type
      });
    }
  }
  return self._userPrivilegeColumns;
};


module.exports = EditUserDialog;
